@@ -14,13 +14,19 @@
  */
 /* Multibyte support	Eiji Tokuya 2001-03-15 */
 
+#ifndef	NOT_USE_LIBPQ
 #include <libpq-fe.h>
+#endif /* NOT_USE_LIBPQ */
 #include "connection.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#ifndef	WIN32
+#ifdef	WIN32
+#ifdef	USE_SSPI
+#include "sspisvcs.h"
+#endif /* USE_SSPI */
+#else
 #include <errno.h>
 #endif /* WIN32 */
 
@@ -311,23 +317,16 @@ reset_current_schema(ConnectionClass *self)
 	}
 }
 
+extern	int	exepgm;
 ConnectionClass *
 CC_Constructor()
 {
-	extern	int	exepgm;
 	ConnectionClass *rv, *retrv = NULL;
 
 	rv = (ConnectionClass *) calloc(sizeof(ConnectionClass), 1);
 
 	if (rv != NULL)
 	{
-		// rv->henv = NULL;		/* not yet associated with an environment */
-
-		// rv->__error_message = NULL;
-		// rv->__error_number = 0;
-		// rv->sqlstate[0] = '\0';
-		// rv->errormsg_created = FALSE;
-
 		rv->status = CONN_NOT_CONNECTED;
 		rv->transact_status = CONN_IN_AUTOCOMMIT;		/* autocommit by default */
 
@@ -352,37 +351,14 @@ CC_Constructor()
 #endif /* ODBCVER */
 
 		rv->lobj_type = PG_TYPE_LO_UNDEFINED;
-
-		// rv->ncursors = 0;
-		// rv->ntables = 0;
-		// rv->col_info = NULL;
-
-		// rv->translation_option = 0;
-		// rv->translation_handle = NULL;
-		// rv->DataSourceToDriver = NULL;
-		// rv->DriverToDataSource = NULL;
 		rv->driver_version = ODBCVER;
 #ifdef	WIN32
 		if (VER_PLATFORM_WIN32_WINDOWS == platformId && rv->driver_version > 0x0300)
 			rv->driver_version = 0x0300;
 #endif /* WIN32 */
-		// memset(rv->pg_version, 0, sizeof(rv->pg_version));
-		// rv->pg_version_number = .0;
-		// rv->pg_version_major = 0;
-		// rv->pg_version_minor = 0;
-		// rv->ms_jet = 0;
 		if (1 == exepgm)
 			rv->ms_jet = 1;
-		// rv->unicode = 0;
-		// rv->result_uncommitted = 0;
-		// rv->schema_support = 0;
 		rv->isolation = SQL_TXN_READ_COMMITTED;
-		// rv->original_client_encoding = NULL;
-		// rv->current_client_encoding = NULL;
-		// rv->server_encoding = NULL;
-		// rv->current_schema = NULL;
-		// rv->num_discardp = 0;
-		// rv->discardp = NULL;
 		rv->mb_maxbyte_per_char = 1;
 		rv->max_identifier_length = -1;
 		rv->escape_in_literal = ESCAPE_IN_LITERAL;
@@ -814,25 +790,36 @@ handle_error_message(ConnectionClass *self, char *msgbuf, size_t buflen, char *s
 {
 	BOOL	new_format = FALSE, msg_truncated = FALSE, truncated, hasmsg = FALSE;
 	SocketClass	*sock = self->sock;
+	ConnInfo	*ci = &(self->connInfo);
 	char	msgbuffer[ERROR_MSG_LENGTH];
 	UDWORD	abort_opt;
 
-inolog("handle_error_message prptocol=%s\n", self->connInfo.protocol);
-	if (PROTOCOL_74(&(self->connInfo)))
+	inolog("handle_error_message protocol=%s\n", ci->protocol);
+	if (PROTOCOL_74(ci))
 		new_format = TRUE;
+	else if (PROTOCOL_74REJECTED(ci))
+	{
+		if (!SOCK_get_next_byte(sock, TRUE)) /* peek the next byte */
+		{
+			uint32	leng;
+
+			mylog("peek the next byte = \\0\n");
+			new_format = TRUE;
+			strncpy(ci->protocol, PG74, sizeof(ci->protocol));
+			leng = SOCK_get_response_length(sock);
+			inolog("get the response length=%d\n", leng);
+		}
+	}
 
 inolog("new_format=%d\n", new_format);
+	truncated = SOCK_get_string(sock, new_format ? msgbuffer : msgbuf, new_format ? sizeof(msgbuffer) : buflen);
 	if (new_format)
 	{
 		size_t	msgl;
 
 		msgbuf[0] = '\0';
-		for (;;)
+		for (;msgbuffer[0];)
 		{
-			truncated = SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
-			if (!msgbuffer[0])
-				break;
-
 			mylog("%s: 'E' - %s\n", comment, msgbuffer);
 			qlog("ERROR from backend during %s: '%s'\n", comment, msgbuffer);
 			msgl = strlen(msgbuffer + 1);
@@ -878,23 +865,23 @@ inolog("new_format=%d\n", new_format);
 				buflen = 0;
 			while (truncated)
 				truncated = SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
+			truncated = SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
 		}
 	}
 	else
 	{
-		msg_truncated = SOCK_get_string(sock, msgbuf, (Int4) buflen);
-
+		msg_truncated = truncated;
 		/* Remove a newline */
 		if (msgbuf[0] != '\0' && msgbuf[(int)strlen(msgbuf) - 1] == '\n')
 			msgbuf[(int)strlen(msgbuf) - 1] = '\0';
 
 		mylog("%s: 'E' - %s\n", comment, msgbuf);
 		qlog("ERROR from backend during %s: '%s'\n", comment, msgbuf);
-		for (truncated = msg_truncated; truncated;)
+		while (truncated)
 			truncated = SOCK_get_string(sock, msgbuffer, sizeof(msgbuffer));
 	}
 	abort_opt = 0;
-	if (!strncmp(msgbuffer, "FATAL", 5))
+	if (!strncmp(msgbuf, "FATAL", 5))
 	{
 		CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_ERROR);
 		abort_opt = CONN_DEAD;
@@ -1345,6 +1332,7 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 }
 
 static	char	CC_setenv(ConnectionClass *self);
+#ifndef	NOT_USE_LIBPQ
 static int LIBPQ_connect(ConnectionClass *self);
 static char
 LIBPQ_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
@@ -1366,6 +1354,7 @@ LIBPQ_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 
 	return 1;
 }
+#endif /* NOT_USE_LIBPQ */
 
 static char
 original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
@@ -1380,8 +1369,11 @@ original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 	char		msgbuffer[ERROR_MSG_LENGTH];
 	char		salt[5], notice[512];
 	CSTR		func = "original_CC_connect";
-	// char	   *encoding;
-	BOOL	startPacketReceived = FALSE;
+	BOOL	startPacketReceived = FALSE, anotherVersionRetry;
+#ifdef	USE_SSPI
+	int	ssl_try_count, ssl_try_no;
+	char	ssl_call[2];
+#endif /* USE_SSPI */
 
 	mylog("%s: entering...\n", func);
 
@@ -1395,8 +1387,69 @@ original_CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 		if (0 == CC_initial_log(self, func))
 			return 0;
 
+#ifdef	USE_SSPI
+		ssl_try_count = 0;
+		switch (self->connInfo.sslmode[0])
+		{
+			case 'r':
+				ssl_call[ssl_try_count++] = 'y';
+				break;
+			case 'p':
+				ssl_call[ssl_try_count++] = 'y';
+				ssl_call[ssl_try_count++] = 'n';
+				break;
+			case 'a':
+				ssl_call[ssl_try_count++] = 'n';
+				ssl_call[ssl_try_count++] = 'y';
+				break;
+			default:
+				ssl_call[ssl_try_count++] = 'n';
+				break;
+		}
+		ssl_try_no = 0;
+#endif /* USE_SSPI */
+		anotherVersionRetry = FALSE;
+
 another_version_retry:
 
+		if (anotherVersionRetry)
+		{
+#ifdef	USE_SSPI
+			if (PROTOCOL_74(ci) || PROTOCOL_64(ci))
+			{
+				if (ssl_try_no < ssl_try_count)
+					ssl_try_no++;
+			}
+			else
+				ssl_try_no = ssl_try_count;
+			if (ssl_try_no >= ssl_try_count)
+			{
+#endif /* USE_SSPI */
+				/* retry older version */
+				if (PROTOCOL_62(ci))
+				{
+					CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, "Could not construct a socket to the server", func);
+					return 0;
+				}
+				if (PROTOCOL_63(ci))
+					strncpy(ci->protocol, PG62, sizeof(ci->protocol));
+				else if (PROTOCOL_64(ci))
+					strncpy(ci->protocol, PG63, sizeof(ci->protocol));
+				else 
+					strncpy(ci->protocol, PG64, sizeof(ci->protocol));
+#ifdef	USE_SSPI
+				ssl_try_no = 0;
+			}
+#endif /* USE_SSPI */
+			if (self->sock)
+			{
+				SOCK_Destructor(self->sock);
+				self->sock = (SocketClass *) 0;
+			}
+			self->status = CONN_NOT_CONNECTED;
+			CC_initialize_pg_version(self);
+			anotherVersionRetry = FALSE;
+		}
 		/*
 		 * If the socket was closed for some reason (like a SQLDisconnect,
 		 * but no SQLFreeConnect then create a socket now.
@@ -1424,6 +1477,53 @@ another_version_retry:
 		mylog("connection to the server socket succeeded.\n");
 
 inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self->pg_version_minor);
+#ifdef	USE_SSPI
+		if ('y' == ssl_call[ssl_try_no])
+		{
+			struct {
+				Int4	slen;
+				ProtocolVersion	pv;
+			} nego_packet;
+			char	rnego;
+
+			nego_packet.slen = htonl(sizeof(nego_packet));
+			nego_packet.pv = htonl(PG_NEGOTIATE_SSLMODE);
+			SOCK_put_n_char(sock, (char *) &nego_packet, sizeof(nego_packet));
+			SOCK_flush_output(sock);
+			if (0 != SOCK_get_errcode(sock))
+			{
+				mylog("%s:failed to send a negotiation packet\n", func);
+				goto sockerr_proc;
+			}
+			SOCK_get_n_char(sock, &rnego, 1);
+			if (0 != SOCK_get_errcode(sock))
+				goto sockerr_proc;
+			mylog("nego got '%c'\n", rnego);
+			switch (rnego)
+			{
+				case 'S':
+					if (!StartupSspiService(sock, SchannelService, NULL))
+					{
+						CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, "Service negotation failed", func);
+						return 0;
+					}
+					break;
+				case 'N':
+					ssl_try_no++;
+					if (ssl_try_no < ssl_try_count &&
+					    'y' != ssl_call[ssl_try_no])
+						break;
+					else
+					{
+						anotherVersionRetry = TRUE;
+						goto another_version_retry;
+					}
+				default:
+					CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, "Service negotation failed", func);
+					return 0;
+			}
+		}
+#endif /* USE_SSPI */
 		if (PROTOCOL_62(ci))
 		{
 			sock->reverse = TRUE;		/* make put_int and get_int work
@@ -1505,13 +1605,12 @@ inolog("protocol=%s version=%d,%d\n", ci->protocol, self->pg_version_major, self
 					if (beresp != 'E' || startPacketReceived)
 					{
 						leng = SOCK_get_response_length(sock);
-inolog("leng=%d\n", leng);
+						inolog("leng=%d\n", leng);
 						if (0 != SOCK_get_errcode(sock))
 							goto sockerr_proc;
 					}
 					else
 						strncpy(ci->protocol, PG74REJECTED, sizeof(ci->protocol));
-/* retry = TRUE; */
 				}
 				startPacketReceived = TRUE;
 			}
@@ -1519,14 +1618,24 @@ inolog("leng=%d\n", leng);
 			switch (beresp)
 			{
 				case 'E':
-inolog("Ekita\n");
+inolog("Ekita retry=%d\n", retry);
 					handle_error_message(self, msgbuffer, sizeof(msgbuffer), self->sqlstate, func, NULL);
 					CC_set_error(self, CONN_INVALID_AUTHENTICATION, msgbuffer, func);
 					qlog("ERROR from backend during authentication: '%s'\n", msgbuffer);
-					if (strnicmp(msgbuffer, "Unsupported frontend protocol", 29) == 0)
+					if (PROTOCOL_74REJECTED(ci))
 						retry = TRUE;
-					else if (strncmp(msgbuffer, "FATAL:", 6) == 0 &&
-						 strnicmp(msgbuffer + 8, "unsupported frontend protocol", 29) == 0)
+					else if (0 == strncmp(msgbuffer, "FATAL:", 6))
+					{
+						const char *emsg = msgbuffer + 8;
+						if (0 == strnicmp(emsg, "unsupported frontend protocol", 29))
+							retry = TRUE;
+#ifdef	USE_SSPI
+						else if ('y' != ssl_call[ssl_try_no] &&
+							 ssl_try_no + 1 < ssl_try_count)
+							retry = TRUE;
+#endif /* USE_SSPI */
+					}
+					else if (strnicmp(msgbuffer, "Unsupported frontend protocol", 29) == 0)
 						retry = TRUE;
 					if (retry)
 						break;
@@ -1637,16 +1746,8 @@ inolog("Ekita\n");
 					return 0;
 			}
 			if (retry)
-			{	/* retry older version */
-				if (PROTOCOL_63(ci))
-					strncpy(ci->protocol, PG62, sizeof(ci->protocol));
-				else if (PROTOCOL_64(ci))
-					strncpy(ci->protocol, PG63, sizeof(ci->protocol));
-				else 
-					strncpy(ci->protocol, PG64, sizeof(ci->protocol));
-				SOCK_Destructor(sock);
-				self->sock = (SocketClass *) 0;
-				CC_initialize_pg_version(self);
+			{
+				anotherVersionRetry = TRUE;
 				goto another_version_retry;
 			}
 
@@ -1688,7 +1789,7 @@ sockerr_proc:
 		{
 			CC_set_error(self, CONNECTION_NO_SUCH_DATABASE, "The database does not exist on the server\nor user authentication failed.", func);
 			QR_Destructor(res);
-		return 0;
+			return 0;
 		}
 		QR_Destructor(res);
 
@@ -1710,35 +1811,54 @@ inolog("CC_lookup_pg_version\n");
 char
 CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 {
-	// StartupPacket sp;
-	// StartupPacket6_2 sp62;
-	// QResultClass *res;
-	// SocketClass *sock;
-	ConnInfo   *ci = &(self->connInfo);
-	// int			areq = -1;
-	// int			beresp;
-	// char		msgbuffer[ERROR_MSG_LENGTH];
-	// char		salt[5], notice[512];
-	CSTR		func = "CC_connect";
-	char	   ret;
-	// char	   *encoding;
-	// BOOL	startPacketReceived = FALSE;
+	ConnInfo *ci = &(self->connInfo);
+	CSTR	func = "CC_connect";
+	char		ret, *saverr = NULL;
+#ifndef	NOT_USE_LIBPQ
+	BOOL	call_libpq = FALSE;
+#endif /* NOT_USE_LIBPQ */
 
 	mylog("%s: entering...\n", func);
 
 	mylog("sslmode=%s\n", self->connInfo.sslmode);
-	if (self->connInfo.sslmode[0] != 'd' ||
-	    self->connInfo.username[0] == '\0')
+#ifndef	NOT_USE_LIBPQ
+	if (self->connInfo.username[0] == '\0')
+		call_libpq = TRUE;
+	else if (self->connInfo.sslmode[0] != 'd') 
+#ifdef	USE_SSPI
+	{
+		if (0 == (SchannelService & self->svcs_allowed))
+			call_libpq = TRUE;
+	}
+#else
+		call_libpq = TRUE;
+#endif /* USE_SSPI */
+	if (call_libpq)
+	{
 		ret = LIBPQ_CC_connect(self, password_req, salt_para);
+#ifdef USE_SSPI
+		/*
+		 *	If libpq is unavailable, try SSPI instead.
+		 */
+		if (0 == ret && CONN_UNABLE_TO_LOAD_DLL == CC_get_errornumber(self))
+		{
+			self->svcs_allowed |= SchannelService;
+			ret = original_CC_connect(self, password_req, salt_para);
+		}
+#endif /* USE_SSPI */
+	}
 	else
+#endif /* NOT_USE_LIBPQ */
 	{
 		ret = original_CC_connect(self, password_req, salt_para);
+#ifndef	NOT_USE_LIBPQ
 		if (0 == ret && CONN_AUTH_TYPE_UNSUPPORTED == CC_get_errornumber(self))
 		{
 			SOCK_Destructor(self->sock);
 			self->sock = (SocketClass *) 0;
 			ret = LIBPQ_CC_connect(self, password_req, salt_para);
 		}
+#endif /* NOT_USE_LIBPQ */
 	}
 	if (ret <= 0)
 		return ret;
@@ -1758,6 +1878,8 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 inolog("CC_send_settings\n");
 	CC_send_settings(self);
 
+	if (CC_get_errornumber(self) > 0)
+		saverr = strdup(CC_get_errormsg(self));
 	CC_clear_error(self);			/* clear any error */
 	CC_lookup_lo(self);			/* a hack to get the oid of
 						   our large object oid type */
@@ -1769,7 +1891,10 @@ inolog("CC_send_settings\n");
 	{
 		CC_lookup_characterset(self);
 		if (CC_get_errornumber(self) > 0)
-			return 0;
+		{
+			ret = 0;
+			goto cleanup;
+		}
 #ifdef UNICODE_SUPPORT
 		if (CC_is_in_unicode_driver(self))
 		{
@@ -1780,7 +1905,8 @@ inolog("CC_send_settings\n");
 				if (PG_VERSION_LT(self, 7.1))
 				{
 					CC_set_error(self, CONN_NOT_IMPLEMENTED_ERROR, "UTF-8 conversion isn't implemented before 7.1", func);
-					return 0;
+					ret = 0;
+					goto cleanup;
 				}
 				if (self->original_client_encoding)
 					free(self->original_client_encoding);
@@ -1802,7 +1928,8 @@ inolog("CC_send_settings\n");
 	else if (CC_is_in_unicode_driver(self))
 	{
 		CC_set_error(self, CONN_NOT_IMPLEMENTED_ERROR, "Unicode isn't supported before 6.4", func);
-		return 0;
+		ret = 0;
+		goto cleanup;
 	}
 #endif /* UNICODE_SUPPORT */
 	ci->updatable_cursors = DISALLOW_UPDATABLE_CURSORS; 
@@ -1825,10 +1952,18 @@ inolog("CC_send_settings\n");
 	    && 0 < ci->bde_environment)
 		self->unicode |= CONN_DISALLOW_WCHAR;
 mylog("conn->unicode=%d\n", self->unicode);
+	ret = 1;
 
-	mylog("%s: returning...\n", func);
+cleanup:
+	mylog("%s: returning...%d\n", func, ret);
+	if (NULL != saverr)
+	{
+		if (ret > 0 && CC_get_errornumber(self) <= 0)
+			CC_set_error(self, -1, saverr, func);
+		free(saverr);
+	}
 
-	return 1;
+	return ret;
 }
 
 
@@ -2031,7 +2166,7 @@ static void CC_clear_cursors(ConnectionClass *self, BOOL on_abort)
 	{
 		stmt = self->stmts[i];
 		if (stmt && (res = SC_get_Result(stmt)) &&
-			QR_get_cursor(res))
+			 (NULL != QR_get_cursor(res)))
 		{
 			if ((on_abort && !QR_is_permanent(res)) ||
 				!QR_is_withhold(res))
@@ -2206,6 +2341,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 
 	/* QR_set_command() dups this string so doesn't need static */
 	char		cmdbuffer[ERROR_MSG_LENGTH + 1];
+	BOOL		reduce_round_trip_time = !(flag & IGNORE_ROUND_TRIP);
 
 	if (appendq)
 	{
@@ -2244,6 +2380,28 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 		return NULL;
 	}
 
+	/*
+	 *	In case the round trip time can be ignored, the query
+	 *	and the appeneded query would be issued separately.
+	 *	Otherwise a multiple command query would be issued.
+	 */ 
+	if (appendq && !reduce_round_trip_time)
+	{
+		res = CC_send_query_append(self, query, qi, flag, stmt, NULL);
+		if (QR_command_maybe_successful(res))
+		{
+			cmdres = CC_send_query_append(self, appendq, qi, flag & (~(GO_INTO_TRANSACTION)), stmt, NULL);
+			if (QR_command_maybe_successful(cmdres))
+				res->next = cmdres;
+			else
+			{
+				QR_Destructor(res);
+				res = cmdres;
+			}
+		}
+		return res;
+	}
+
 	rollback_on_error = (flag & ROLLBACK_ON_ERROR) != 0;
 	end_with_commit = (flag & END_WITH_COMMIT) != 0;
 #define	return DONT_CALL_RETURN_FROM_HERE???
@@ -2734,7 +2892,8 @@ cleanup:
 					CC_set_errornumber(self, 0);
 				else if (retres)
 				{
-					if ((!CC_get_errormsg(self) || !CC_get_errormsg(self)[0]))
+					if (NULL == CC_get_errormsg(self) ||
+					    !CC_get_errormsg(self)[0])
 						CC_set_errormsg(self, QR_get_message(retres));
 					if (!self->sqlstate[0])
 						strcpy(self->sqlstate, retres->sqlstate);
@@ -3352,8 +3511,10 @@ CC_send_cancel_request(const ConnectionClass *conn)
 	if (!sock)
 		return FALSE;
 
+#ifndef NOT_USE_LIBPQ
 	if (sock->via_libpq)
 		return LIBPQ_send_cancel_request(conn);
+#endif /* NOT_USE_LIBPQ */
 	/*
 	 * We need to open a temporary connection to the postmaster. Use the
 	 * information saved by connectDB to do this with only kernel calls.
@@ -3441,6 +3602,7 @@ int	CC_discard_marked_objects(ConnectionClass *conn)
 	return 1;
 }
 
+#ifndef NOT_USE_LIBPQ
 static int
 LIBPQ_connect(ConnectionClass *self)
 {
@@ -3474,7 +3636,7 @@ inolog("sock=%p\n", sock);
 	free(conninfo);
 	if (!libpqLoaded)
 	{
-		CC_set_error(self, CONN_OPENDB_ERROR, "Couldn't load libpq library", func);
+		CC_set_error(self, CONN_UNABLE_TO_LOAD_DLL, "Couldn't load libpq library", func);
 		goto cleanup1;
 	}
 	sock->via_libpq = TRUE;
@@ -3602,10 +3764,18 @@ LIBPQ_send_cancel_request(const ConnectionClass *conn)
 	else
 		return FALSE;
 }
+#endif /* NOT_USE_LIBPQ */
 
 const char *CurrCat(const ConnectionClass *conn)
 {
-	if (conn->schema_support)
+	/*
+	 * Returning the database name causes problems in MS Query. It
+	 * generates query like: "SELECT DISTINCT a FROM byronnbad3
+	 * bad3"
+	 */
+	if (2 == exepgm)	/* MS Query */
+		return NULL;
+	else if (conn->schema_support)
 		return conn->connInfo.database;
 	else
 		return NULL;
