@@ -14,15 +14,19 @@
 #ifndef	WIN32
 #include <errno.h>
 #endif /* WIN32 */
+#ifndef NOT_USE_LIBPQ
 #include <libpq-fe.h>
+#endif /* NOT_USE_LIBPQ */
 #include "loadlib.h"
 #include "pgenlist.h"
 
 #ifdef  WIN32
 #ifdef  _MSC_VER
 #pragma comment(lib, "Delayimp")
+#ifndef	NOT_USE_LIBPQ
 #pragma comment(lib, "libpq")
 #pragma comment(lib, "ssleay32")
+#endif /* NOT_USE_LIBPQ */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 #ifdef	UNICODE_SUPPORT
 #pragma comment(lib, "pgenlist")
@@ -33,8 +37,10 @@
 // The followings works under VC++6.0 but doesn't work under VC++7.0.
 // Please add the equivalent linker options using command line etc.
 #if (_MSC_VER == 1200) && defined(DYNAMIC_LOAD) // VC6.0
+#ifndef	NOT_USE_LIBPQ
 #pragma comment(linker, "/Delayload:libpq.dll")
 #pragma comment(linker, "/Delayload:ssleay32.dll")
+#endif /* NOT_USE_LIBPQ */
 #ifdef	UNICODE_SUPPORT
 #pragma comment(linker, "/Delayload:pgenlist.dll")
 #else
@@ -129,7 +135,8 @@ DliErrorHook(unsigned	dliNotify,
 				if (hmodule = MODULE_load_from_psqlodbc_path(pgenlist), NULL == hmodule)
 					hmodule = LoadLibrary(pgenlist);
 			}
-			else
+			else if (0 == _stricmp(pdli->szDll, libarray[0]) ||
+				 0 == _stricmp(pdli->szDll, libarray[1]))
 			{
         			mylog("getting alternative ssl library instead of %s\n", pdli->szDll);
         			for (i = 0; i < sizeof(libarray) / sizeof(const char * const); i++)
@@ -187,6 +194,7 @@ void CleanupDelayLoadedDLLs(void)
 }
 #endif	/* _MSC_DELAY_LOAD_IMPORT */
 
+#ifndef	NOT_USE_LIBPQ
 void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 {
 	void *pqconn = NULL;
@@ -214,15 +222,16 @@ void *CALL_PQconnectdb(const char *conninfo, BOOL *libpqLoaded)
 	{
 		loaded_libpq = TRUE;
 		/* ssllibs are already loaded by libpq
-		if (PQgetssl(pqconn))
+		if (PQgetssl(pqconn)) */
 			loaded_ssllib = TRUE;
-		*/
+		
 	}
 #else
 	pqconn = PQconnectdb(conninfo);
 #endif /* _MSC_DELAY_LOAD_IMPORT */
 	return pqconn;
 }
+#endif /* NOT_USE_LIBPQ */
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 RETCODE	CALL_EnlistInDtc(ConnectionClass *conn, void *pTra, int method)
@@ -272,23 +281,32 @@ RETCODE	CALL_DtcOnRelease(void)
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 
 #if defined(WIN_DYN_LOAD)
-BOOL LIBPQ_check()
+BOOL SSLLIB_check()
 {
 	extern	HINSTANCE	s_hModule;
 	HMODULE	hmodule = NULL;
 
 	mylog("checking libpq library\n");
 	/* First search the driver's folder */
+#ifndef	NOT_USE_LIBPQ
 	if (NULL == (hmodule = MODULE_load_from_psqlodbc_path(libpq)))
 		/* Second try the PATH ordinarily */
 		hmodule = LoadLibrary(libpq);
-	mylog("hmodule=%p\n", hmodule);
+	mylog("libpq hmodule=%p\n", hmodule);
+#endif /* NOT_USE_LIBPQ */
+#ifdef	USE_SSPI
+	if (NULL == hmodule)
+	{
+		hmodule = LoadLibrary("secur32.dll");
+		mylog("secur32 hmodule=%p\n", hmodule);
+	}
+#endif /* USE_SSPI */
 	if (hmodule)
 		FreeLibrary(hmodule);
 	return (NULL != hmodule);
 }
 #else
-BOOL LIBPQ_check()
+BOOL SSLLIB_check()
 {
 	return TRUE;
 }
