@@ -358,7 +358,8 @@ pgtype_to_concise_type(StatementClass *stmt, OID type, int col)
 			return SQL_FLOAT;
 		case PG_TYPE_BOOL:
 			return ci->drivers.bools_as_char ? SQL_CHAR : SQL_BIT;
-
+		case PG_TYPE_XML:
+			return CC_is_in_unicode_driver(conn) ? SQL_WLONGVARCHAR : SQL_LONGVARCHAR;
 		default:
 
 			/*
@@ -1142,7 +1143,7 @@ pgtype_transfer_octet_length(StatementClass *stmt, OID type, int col, int handle
 #endif /* UNICODE_SUPPORT */
 			/* after 7.2 */
 			if (PG_VERSION_GE(conn, 7.2))
-				coef =conn->mb_maxbyte_per_char;
+				coef = conn->mb_maxbyte_per_char;
 			if (coef < 2 && (conn->connInfo).lf_conversion)
 				/* CR -> CR/LF */
 				coef = 2;
