@@ -23,9 +23,6 @@
 #include "environ.h"
 
 #ifdef WIN32
-#ifdef _WSASTARTUP_IN_DLLMAIN_
-#include <winsock2.h>
-#endif /* _WSASTARTUP_IN_DLLMAIN_ */
 #include "loadlib.h"
 int	platformId = 0;
 #endif
@@ -109,31 +106,11 @@ HINSTANCE NEAR s_hModule;		/* Saved module handle. */
 BOOL		WINAPI
 DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
 {
-#ifdef _WSASTARTUP_IN_DLLMAIN_
-	WORD		wVersionRequested;
-	WSADATA		wsaData;
-#endif /* _WSASTARTUP_IN_DLLMAIN_ */
-
 	switch (ul_reason_for_call)
 	{
 		case DLL_PROCESS_ATTACH:
 			s_hModule = hInst;	/* Save for dialog boxes */
 
-#ifdef	_WSASTARTUP_IN_DLLMAIN_
-			/* Load the WinSock Library */
-			wVersionRequested = MAKEWORD(1, 1);
-
-			if (WSAStartup(wVersionRequested, &wsaData))
-				return FALSE;
-
-			/* Verify that this is the minimum version of WinSock */
-			if (LOBYTE(wsaData.wVersion) != 1 ||
-				HIBYTE(wsaData.wVersion) != 1)
-			{
-				WSACleanup();
-				return FALSE;
-			}
-#endif /* _WSASTARTUP_IN_DLLMAIN_ */
 			if (initialize_global_cs() == 0)
 			{
 				char	pathname[_MAX_PATH], fname[_MAX_FNAME];
@@ -145,6 +122,8 @@ DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
 					_splitpath(pathname, NULL, NULL, fname, NULL);
 					if (stricmp(fname, "msaccess") == 0)
 						exepgm = 1;
+					else if (strnicmp(fname, "msqry", 5) == 0)
+						exepgm = 2;
 				}
 				osversion.dwOSVersionInfoSize = sizeof(osversion);
 				if (GetVersionEx(&osversion))
@@ -163,9 +142,6 @@ DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
 			CleanupDelayLoadedDLLs();
 			/* my(q)log is unavailable from here */
 			finalize_global_cs();
-#ifdef	_WSASTARTUP_IN_DLLMAIN_
-			WSACleanup();
-#endif /* _WSASTARTUP_IN_DLLMAIN_ */
 			return TRUE;
 
 		case DLL_THREAD_DETACH:
