@@ -1663,6 +1663,7 @@ inolog("curt=%d\n", curt);
 }
 
 
+#include "dlg_specific.h"
 RETCODE
 SC_execute(StatementClass *self)
 {
@@ -1818,6 +1819,8 @@ inolog("get_Result=%p\n", res);
 			sprintf(fetch, "%s " FORMAT_LEN " in \"%s\"", fetch_cmd, qi.row_size, SC_cursor_name(self));
 			qryi = &qi;
 			appendq = fetch;
+			if (0 != (ci->extra_opts & BIT_IGNORE_ROUND_TRIP_TIME))
+				qflag |= IGNORE_ROUND_TRIP;
 		}
 		res = CC_send_query_append(conn, self->stmt_with_params, qryi, qflag, SC_get_ancestor(self), appendq);
 		if (SC_is_fetchcursor(self) && QR_command_maybe_successful(res))
